@@ -263,7 +263,7 @@ impl Iterator for ResolvePath {
             };
             self.steps_remaining -= 1;
 
-            // ..and add that component to the our resolved path to see what we
+            // ..and add that component to our resolved path to see what we
             // should inspect next.
             let inspecting: std::borrow::Cow<'_, Path> = if next_part.text == "." {
                 // Do nothing.
@@ -391,6 +391,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
