@@ -367,14 +367,14 @@ define_accessor_trait! {
         circuit_timing: CircuitTiming,
         preemptive_circuits: PreemptiveCircuitConfig,
         +
-        // TODO HS-VANGUARDS: ideally this would be defined in the same way as `path_rules`,
+        // Note: ideally this would be defined in the same way as `path_rules`,
         // `circuit_timing`, etc., but define_accessor_trait unconditionally adds
         // AsRef<VanguardsConfig> as a supertrait, which can't be cfg'd behind
         // the vanguards feature.
 
         /// Access the field
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-        fn vanguard_config(&self) -> &tor_guardmgr::vanguards::VanguardConfig;
+        fn vanguard_config(&self) -> &tor_guardmgr::VanguardConfig;
     }
 }
 
@@ -385,22 +385,18 @@ pub(crate) mod test_config {
     use crate::*;
     use tor_guardmgr::bridge::BridgeConfig;
     #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-    use tor_guardmgr::vanguards::VanguardConfig;
+    use tor_guardmgr::VanguardConfig;
 
     /// Testing configuration, with public fields
     #[derive(Default, derive_more::AsRef)]
     #[allow(clippy::exhaustive_structs)]
+    #[allow(missing_docs)]
     #[cfg_attr(docsrs, doc(cfg(feature = "testing")))]
     pub struct TestConfig {
-        ///
         pub path_rules: PathConfig,
-        ///
         pub circuit_timing: CircuitTiming,
-        ///
         pub preemptive_circuits: PreemptiveCircuitConfig,
-        ///
         pub guardmgr: tor_guardmgr::TestConfig,
-        ///
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
         pub vanguard_config: VanguardConfig,
     }
@@ -430,7 +426,7 @@ pub(crate) mod test_config {
             &self.preemptive_circuits
         }
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-        fn vanguard_config(&self) -> &tor_guardmgr::vanguards::VanguardConfig {
+        fn vanguard_config(&self) -> &tor_guardmgr::VanguardConfig {
             &self.vanguard_config
         }
     }
