@@ -282,7 +282,9 @@ pub(crate) trait Store: Send + 'static {
     /// Mark the consensus generated from `cmeta` as no longer pending.
     fn mark_consensus_usable(&mut self, cmeta: &ConsensusMeta) -> Result<()>;
     /// Remove the consensus generated from `cmeta`.
-    #[allow(dead_code)] // TODO #1383 do we want to keep this anyway
+    //
+    // Nothing uses this yet; removal is handled from `expire_all`.
+    #[allow(dead_code)]
     fn delete_consensus(&mut self, cmeta: &ConsensusMeta) -> Result<()>;
 
     /// Read all of the specified authority certs from the cache.
@@ -330,13 +332,14 @@ pub(crate) trait Store: Send + 'static {
     ///
     /// It's not an error if it's not present.
     #[cfg(feature = "bridge-client")]
-    #[allow(dead_code)] // TODO #1383 is lack of call sites indication of a bug?
+    // Nothing uses this yet; removal is handled from `expire_all`.
+    #[allow(dead_code)]
     fn delete_bridgedesc(&mut self, bridge: &BridgeConfig) -> Result<()>;
 }
 
 /// Value in the bridge descriptor cache
 #[derive(Clone, Debug)]
-#[cfg_attr(not(feature = "bridge_client"), allow(dead_code))]
+#[cfg_attr(not(feature = "bridge-client"), allow(dead_code))]
 pub(crate) struct CachedBridgeDescriptor {
     /// When we fetched this
     pub(crate) fetched: SystemTime,
