@@ -1,8 +1,8 @@
 #![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
-#![cfg_attr(not(ci_arti_stable), allow(renamed_and_removed_lints))]
-#![cfg_attr(not(ci_arti_nightly), allow(unknown_lints))]
+#![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
+#![allow(unknown_lints)] // @@REMOVE_WHEN(ci_arti_nightly)
 #![warn(missing_docs)]
 #![warn(noop_method_call)]
 #![warn(unreachable_pub)]
@@ -62,9 +62,9 @@ use tor_proto::ClockSkew;
 use tor_units::BoundedInt32;
 use tracing::{debug, info, trace, warn};
 
-use tor_config::impl_standard_builder;
-use tor_config::ReconfigureError;
 use tor_config::{define_list_builder_accessors, define_list_builder_helper};
+use tor_config::{impl_not_auto_value, ReconfigureError};
+use tor_config::{impl_standard_builder, ExplicitOrAuto};
 use tor_netdir::{params::NetParameters, NetDir, Relay};
 use tor_persist::{DynStorageHandle, StateMgr};
 use tor_rtcompat::Runtime;
@@ -114,6 +114,8 @@ use sample::{GuardSet, Universe, UniverseRef};
 
 use crate::ids::{FirstHopIdInner, GuardId};
 
+use tor_config::ConfigBuildError;
+
 /// A "guard manager" that selects and remembers a persistent set of
 /// guard nodes.
 ///
@@ -1870,7 +1872,6 @@ impl VanguardMode {
     /// Used for converting [`vanguards_enabled`](NetParameters::vanguards_enabled)
     /// or [`vanguards_hs_service`](NetParameters::vanguards_hs_service)
     /// to the corresponding `VanguardMode`.
-    #[allow(dead_code)] // TODO HS-VANGUARDS
     pub(crate) fn from_net_parameter(val: BoundedInt32<0, 2>) -> Self {
         match val.get() {
             0 => VanguardMode::Disabled,
@@ -1881,6 +1882,32 @@ impl VanguardMode {
     }
 }
 
+impl_not_auto_value!(VanguardMode);
+
+/// Vanguards configuration.
+#[derive(Debug, Default, Clone, Eq, PartialEq, derive_builder::Builder)]
+#[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Debug, Serialize, Deserialize))]
+pub struct VanguardConfig {
+    /// The kind of vanguards to use.
+    #[builder_field_attr(serde(default))]
+    #[builder(default)]
+    mode: ExplicitOrAuto<VanguardMode>,
+}
+
+impl VanguardConfig {
+    /// Return the configured [`VanguardMode`].
+    ///
+    /// Returns the [`Default`] `VanguardMode`
+    /// if the mode is [`Auto`](ExplicitOrAuto) or unspecified.
+    pub fn mode(&self) -> VanguardMode {
+        match self.mode {
+            ExplicitOrAuto::Auto => Default::default(),
+            ExplicitOrAuto::Explicit(mode) => mode,
+        }
+    }
+}
+
 /// The kind of vanguards to use.
 #[derive(Debug, Default, Clone, Copy, Eq, PartialEq, Ord, PartialOrd)] //
 #[derive(Serialize, Deserialize)] //
