@@ -3,30 +3,11 @@
 //! These requests are yielded on a stream, and the calling code needs to decide
 //! whether to permit or reject them.
 
-use educe::Educe;
-use futures::{Stream, StreamExt};
-use itertools::Itertools;
-use std::sync::Arc;
+use crate::internal_prelude::*;
+
 use tor_cell::relaycell::msg::{Connected, End, Introduce2};
-use tor_hscrypto::{
-    pk::{HsBlindIdKeypair, HsIdKey, HsIdKeypair, HsIntroPtSessionIdKey, HsSvcNtorKeypair},
-    time::TimePeriod,
-    Subcredential,
-};
-use tor_keymgr::{KeyMgr, KeyPath, KeyPathRange, KeySpecifierComponent, KeySpecifierPattern};
-use tor_persist::slug::Slug;
-
-use tor_error::{internal, Bug};
-use tor_proto::{
-    circuit::ClientCirc,
-    stream::{DataStream, IncomingStream, IncomingStreamRequest},
-};
-
-use crate::{
-    keys::BlindIdKeypairSpecifierPattern,
-    svc::rend_handshake::{self, RendCircConnector},
-    ClientError, FatalError, HsIdKeypairSpecifier, HsNickname, IptLocalId,
-};
+use tor_hscrypto::Subcredential;
+use tor_proto::stream::{IncomingStream, IncomingStreamRequest};
 
 /// Request to complete an introduction/rendezvous handshake.
 ///
@@ -93,6 +74,9 @@ pub(crate) struct RendRequestContext {
     /// and prevent replays across sessions.
     pub(crate) kp_hs_ipt_sid: HsIntroPtSessionIdKey,
 
+    /// Configuration for a filter for this service.
+    pub(crate) filter: rend_handshake::RequestFilter,
+
     /// Provider we'll use to find a directory so that we can build a rendezvous
     /// circuit.
     pub(crate) netdir_provider: Arc<dyn tor_netdir::NetDirProvider>,
@@ -235,6 +219,7 @@ impl RendRequest {
             circuit,
         } = intro_request
             .establish_session(
+                self.context.filter.clone(),
                 self.context.circ_pool.clone(),
                 self.context.netdir_provider.clone(),
             )
