@@ -5,6 +5,7 @@ use std::sync::Arc;
 use crate::slug::BadSlug;
 use crate::FsMistrustErrorExt as _;
 use fs_mistrust::anon_home::PathExt as _;
+use tor_basic_utils::PathExt as _;
 use tor_error::{into_bad_api_usage, Bug, ErrorKind};
 
 /// A resource that we failed to access or where we found a problem.
@@ -20,7 +21,7 @@ pub(crate) enum Resource {
         dir: std::path::PathBuf,
     },
     /// A file on disk within our checked directory.
-    #[display(fmt = "{} in {}", "file.display()", "container.anonymize_home()")]
+    #[display(fmt = "{} in {}", "file.display_lossy()", "container.anonymize_home()")]
     File {
         /// The path to the checked directory
         container: std::path::PathBuf,
@@ -208,6 +209,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
