@@ -171,7 +171,12 @@ pub struct ClientCirc {
     /// Don't use this field to send or receive any data, or perform any network
     /// operations for this circuit!  All network operations should be done by
     /// the circuit reactor.
-    channel: Channel,
+    ///
+    /// TODO: This limitation strongly suggests that we have made a mistake somewhere, and should
+    /// not be holding this field in this structure.  Or maybe the object that lets us send/receive
+    /// from a channel should be separate from Channel itself, like how StreamTarget is separate
+    /// from Circuit.
+    channel: Arc<Channel>,
     /// A future that resolves to Cancelled once the reactor is shut down,
     /// meaning that the circuit is closed.
     #[cfg_attr(not(feature = "experimental-api"), allow(dead_code))]
@@ -270,7 +275,16 @@ impl CircParameters {
     }
 }
 
-/// A stream on a particular circuit.
+/// Internal handle, used to implement a stream on a particular circuit.
+///
+/// The reader and the writer for a stream should hold a `StreamTarget` for the stream;
+/// the reader should additionally hold an `mpsc::Receiver` to get
+/// relay messages for the stream.
+///
+/// When all the `StreamTarget`s for a stream are dropped, the Reactor will
+/// close the stream by sending an END message to the other side.
+/// You can close a stream earlier by using [`StreamTarget::close`]
+/// or [`StreamTarget::close_pending`].
 #[derive(Clone, Debug)]
 pub(crate) struct StreamTarget {
     /// Which hop of the circuit this stream is with.
@@ -1010,7 +1024,7 @@ impl PendingClientCirc {
     ///
     pub(crate) fn new(
         id: CircId,
-        channel: Channel,
+        channel: Arc<Channel>,
         createdreceiver: oneshot::Receiver<CreateResponse>,
         input: mpsc::Receiver<ClientCircChanMsg>,
         unique_id: UniqId,
@@ -1262,6 +1276,20 @@ impl StreamTarget {
         Ok(rx)
     }
 
+    /// Queue a "close" for the stream corresponding to this StreamTarget.
+    ///
+    /// Unlike `close_pending`, this method does not allow the caller to provide an `END` message.
+    ///
+    /// Once this method has been called, no more messages may be sent with [`StreamTarget::send`],
+    /// on this `StreamTarget`` or any clone of it.
+    /// The reactor *will* try to flush any already-send messages before it closes the stream.
+    ///
+    /// You don't need to call this method if the stream is closing because all of its StreamTargets
+    /// have been dropped.
+    pub(crate) fn close(&mut self) {
+        self.tx.close_channel();
+    }
+
     /// Called when a circuit-level protocol error has occurred and the
     /// circuit needs to shut down.
     pub(crate) fn protocol_error(&mut self) {
@@ -1281,7 +1309,7 @@ impl StreamTarget {
     }
 
     /// Return a reference to the circuit that this `StreamTarget` is using.
-    #[cfg(feature = "experimental-api")]
+    #[cfg(any(feature = "experimental-api", feature = "stream-ctrl"))]
     pub(crate) fn circuit(&self) -> &Arc<ClientCirc> {
         &self.circ
     }
@@ -1333,7 +1361,9 @@ mod test {
     use tor_basic_utils::test_rng::testing_rng;
     use tor_cell::chancell::{msg as chanmsg, AnyChanCell, BoxedCellBody};
     use tor_cell::relaycell::extend::NtorV3Extension;
-    use tor_cell::relaycell::{msg as relaymsg, AnyRelayMsgOuter, RelayCellFormat, StreamId};
+    use tor_cell::relaycell::{
+        msg as relaymsg, AnyRelayMsgOuter, RelayCellFormat, RelayCmd, RelayMsg as _, StreamId,
+    };
     use tor_linkspec::OwnedCircTarget;
     use tor_rtcompat::{Runtime, SleepProvider};
     use tracing::trace;
@@ -1386,7 +1416,7 @@ mod test {
     fn working_fake_channel<R: Runtime>(
         rt: &R,
     ) -> (
-        Channel,
+        Arc<Channel>,
         Receiver<AnyChanCell>,
         Sender<std::result::Result<OpenChanCellS2C, CodecError>>,
     ) {
@@ -1581,7 +1611,7 @@ mod test {
     // next inbound message seems to come from hop next_msg_from
     async fn newcirc_ext<R: Runtime>(
         rt: &R,
-        chan: Channel,
+        chan: Arc<Channel>,
         next_msg_from: HopNum,
     ) -> (Arc<ClientCirc>, mpsc::Sender<ClientCircChanMsg>) {
         let circid = CircId::new(128).unwrap();
@@ -1626,7 +1656,7 @@ mod test {
     // next inbound message seems to come from hop next_msg_from
     async fn newcirc<R: Runtime>(
         rt: &R,
-        chan: Channel,
+        chan: Arc<Channel>,
     ) -> (Arc<ClientCirc>, mpsc::Sender<ClientCircChanMsg>) {
         newcirc_ext(rt, chan, 2.into()).await
     }
@@ -1992,6 +2022,77 @@ mod test {
         });
     }
 
+    // Test: close a stream, either by dropping it or by calling AsyncWriteExt::close.
+    fn close_stream_helper(by_drop: bool) {
+        tor_rtcompat::test_with_all_runtimes!(|rt| async move {
+            let (chan, mut rx, _sink) = working_fake_channel(&rt);
+            let (circ, mut sink) = newcirc(&rt, chan).await;
+
+            let stream_fut = async move {
+                let stream = circ
+                    .begin_stream("www.example.com", 80, None)
+                    .await
+                    .unwrap();
+
+                let (r, mut w) = stream.split();
+                if by_drop {
+                    // Drop the writer and the reader, which should close the stream.
+                    drop(r);
+                    drop(w);
+                    (None, circ) // make sure to keep the circuit alive
+                } else {
+                    // Call close on the writer, while keeping the reader alive.
+                    w.close().await.unwrap();
+                    (Some(r), circ)
+                }
+            };
+            let handler_fut = async {
+                // Read the BEGIN message.
+                let (_, msg) = rx.next().await.unwrap().into_circid_and_msg();
+                let rmsg = match msg {
+                    AnyChanMsg::Relay(r) => {
+                        AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                            .unwrap()
+                    }
+                    _ => panic!(),
+                };
+                let (streamid, rmsg) = rmsg.into_streamid_and_msg();
+                assert_eq!(rmsg.cmd(), RelayCmd::BEGIN);
+
+                // Reply with a CONNECTED.
+                let connected =
+                    relaymsg::Connected::new_with_addr("10.0.0.1".parse().unwrap(), 1234).into();
+                sink.send(rmsg_to_ccmsg(streamid, connected)).await.unwrap();
+
+                // Expect an END.
+                let (_, msg) = rx.next().await.unwrap().into_circid_and_msg();
+                let rmsg = match msg {
+                    AnyChanMsg::Relay(r) => {
+                        AnyRelayMsgOuter::decode_singleton(RelayCellFormat::V0, r.into_relay_body())
+                            .unwrap()
+                    }
+                    _ => panic!(),
+                };
+                let (_, rmsg) = rmsg.into_streamid_and_msg();
+                assert_eq!(rmsg.cmd(), RelayCmd::END);
+
+                (rx, sink) // keep these alive or the reactor will exit.
+            };
+
+            let ((_opt_reader, _circ), (_rx, _sink)) = futures::join!(stream_fut, handler_fut);
+        });
+    }
+
+    #[test]
+    fn drop_stream() {
+        close_stream_helper(true);
+    }
+
+    #[test]
+    fn close_stream() {
+        close_stream_helper(false);
+    }
+
     // Set up a circuit and stream that expects some incoming SENDMEs.
     async fn setup_incoming_sendme_case<R: Runtime>(
         rt: &R,
