@@ -12,7 +12,7 @@
 //! async fn my_rpc_func(
 //!     target: Arc<OBJTYPE>,
 //!     method: Box<METHODTYPE>,
-//!     ctx: Box<dyn rpc::Context>,
+//!     ctx: Arc<dyn rpc::Context>,
 //!     [ updates: rpc::UpdateSink<METHODTYPE::Update ] // this argument is optional!
 //! ) -> Result<METHODTYPE::Output, impl Into<rpc::RpcError>>
 //! { ... }
@@ -55,6 +55,7 @@ use std::sync::Arc;
 use futures::future::BoxFuture;
 use futures::Sink;
 
+use tor_error::internal;
 use void::Void;
 
 use crate::{Context, DynMethod, Object, RpcError, SendUpdateError};
@@ -72,7 +73,7 @@ pub type RpcResult = Result<RpcValue, RpcError>;
 pub type RpcSendResult = Result<RpcValue, SendUpdateError>;
 
 /// A boxed future holding the result of an RPC method.
-type RpcResultFuture = BoxFuture<'static, RpcResult>;
+pub type RpcResultFuture = BoxFuture<'static, RpcResult>;
 
 /// A boxed sink on which updates can be sent.
 pub type BoxedUpdateSink = Pin<Box<dyn Sink<RpcValue, Error = SendUpdateError> + Send>>;
@@ -116,7 +117,7 @@ pub trait Invocable: Send + Sync + 'static {
         &self,
         obj: Arc<dyn Object>,
         method: Box<dyn DynMethod>,
-        ctx: Box<dyn Context>,
+        ctx: Arc<dyn Context>,
     ) -> Result<SpecialResultFuture, InvokeError>;
 }
 
@@ -130,7 +131,7 @@ pub trait RpcInvocable: Invocable {
         &self,
         obj: Arc<dyn Object>,
         method: Box<dyn DynMethod>,
-        ctx: Box<dyn Context>,
+        ctx: Arc<dyn Context>,
         sink: BoxedUpdateSink,
     ) -> Result<RpcResultFuture, InvokeError>;
 }
@@ -151,7 +152,7 @@ macro_rules! declare_invocable_impl {
       )?
     } => {
         impl<M, OBJ, Fut, S, E, $($update_gen)?> Invocable
-             for fn(Arc<OBJ>, Box<M>, Box<dyn Context + 'static> $(, $update_arg )? ) -> Fut
+             for fn(Arc<OBJ>, Box<M>, Arc<dyn Context + 'static> $(, $update_arg )? ) -> Fut
         where
             M: crate::Method,
             OBJ: Object,
@@ -185,7 +186,7 @@ macro_rules! declare_invocable_impl {
                 &self,
                 obj: Arc<dyn Object>,
                 method: Box<dyn DynMethod>,
-                ctx: Box<dyn Context>,
+                ctx: Arc<dyn Context>,
             ) -> Result<SpecialResultFuture, $crate::InvokeError> {
                 use futures::FutureExt;
                 #[allow(unused)]
@@ -211,7 +212,7 @@ macro_rules! declare_invocable_impl {
         }
 
         impl<M, OBJ, Fut, S, E, $($update_gen)?> RpcInvocable
-            for fn(Arc<OBJ>, Box<M>, Box<dyn Context + 'static> $(, $update_arg )? ) -> Fut
+            for fn(Arc<OBJ>, Box<M>, Arc<dyn Context + 'static> $(, $update_arg )? ) -> Fut
         where
             M: crate::Method,
             M::Output: serde::Serialize,
@@ -229,7 +230,7 @@ macro_rules! declare_invocable_impl {
                 &self,
                 obj: Arc<dyn Object>,
                 method: Box<dyn DynMethod>,
-                ctx: Box<dyn Context>,
+                ctx: Arc<dyn Context>,
                 #[allow(unused)]
                 sink: BoxedUpdateSink,
             ) -> Result<RpcResultFuture, $crate::InvokeError> {
@@ -288,9 +289,17 @@ declare_invocable_impl! {
 #[derive(Clone, Copy)]
 #[must_use]
 pub struct InvokerEnt {
+    /// The function that implements this method on a given type.
+    ///
+    /// Always present.
     #[doc(hidden)]
     pub invoker: &'static (dyn Invocable),
 
+    /// The same function as `invoker`, but only if that function implements
+    /// `RpcInvocable`
+    ///
+    /// This will be `None` if this is a "special" method--that is, one whose inputs and outputs are not serializable,
+    /// and which is therefore not invocable directly from an RPC connection.
     #[doc(hidden)]
     pub rpc_invoker: Option<&'static (dyn RpcInvocable)>,
 
@@ -441,14 +450,14 @@ inventory::collect!(InvokerEnt);
 /// // Note that the types of this function are very constrained:
 /// //  - `obj` must be an Arc<O> for some `Object` type.
 /// //  - `mth` must be Box<M> for some `Method` type.
-/// //  - `ctx` must be Box<dyn rpc::Context>.
+/// //  - `ctx` must be Arc<dyn rpc::Context>.
 /// //  - The function must be async.
 /// //  - The return type must be a Result.
 /// //  - The OK variant of the result must M::Output.
 /// //  - The Err variant of the result must implement Into<rpc::RpcError>.
 /// async fn example(obj: Arc<ExampleObject>,
 ///                  method: Box<ExampleMethod>,
-///                  ctx: Box<dyn rpc::Context>,
+///                  ctx: Arc<dyn rpc::Context>,
 /// ) -> Result<ExampleResult, rpc::RpcError> {
 ///     println!("Running example method!");
 ///     Ok(ExampleResult { text: "here is your result".into() })
@@ -460,7 +469,7 @@ inventory::collect!(InvokerEnt);
 /// // - The fourth argument must be `UpdateSink<M::Update>`.
 /// async fn example2(obj: Arc<ExampleObject2>,
 ///                   method: Box<ExampleMethod>,
-///                   ctx: Box<dyn rpc::Context>,
+///                   ctx: Arc<dyn rpc::Context>,
 ///                   mut updates: rpc::UpdateSink<Progress>
 /// ) -> Result<ExampleResult, rpc::RpcError> {
 ///     updates.send(Progress(0.90)).await?;
@@ -646,27 +655,58 @@ impl DispatchTable {
         ents.into_iter().for_each(|e| self.insert(e));
     }
 
-    /// Try to find an appropriate function for calling a given RPC method on a
-    /// given RPC-visible object.
-    ///
-    /// On success, return a Future.
-    pub fn invoke(
+    /// Helper: Return the invoker for a given RPC object and a given method type,
+    /// if there is one.
+    pub(crate) fn rpc_invoker(
         &self,
-        obj: Arc<dyn Object>,
-        method: Box<dyn DynMethod>,
-        ctx: Box<dyn Context>,
-        sink: BoxedUpdateSink,
-    ) -> Result<RpcResultFuture, InvokeError> {
+        obj: &dyn Object,
+        method: &dyn DynMethod,
+    ) -> Result<&'static dyn RpcInvocable, InvokeError> {
         let func_type = FuncType {
             obj_id: obj.type_id(),
             method_id: method.type_id(),
         };
-
-        let func = self.map.get(&func_type).ok_or(InvokeError::NoImpl)?;
-
-        func.rpc_invoker
+        self.map
+            .get(&func_type)
             .ok_or(InvokeError::NoImpl)?
-            .invoke(obj, method, ctx, sink)
+            .rpc_invoker
+            .ok_or_else(|| {
+                InvokeError::Bug(internal!(
+                    "Somehow tried to call a special method as an RPC method."
+                ))
+            })
+    }
+
+    /// Helper: Return the special invoker for a given object and a given method type,
+    /// if there is one.
+    pub(crate) fn special_invoker<M: crate::Method>(
+        &self,
+        obj: &dyn Object,
+    ) -> Result<&'static dyn Invocable, InvokeError> {
+        let func_type = FuncType {
+            obj_id: obj.type_id(),
+            method_id: std::any::TypeId::of::<M>(),
+        };
+        let ent = self.map.get(&func_type).ok_or(InvokeError::NoImpl)?;
+        Ok(ent.invoker)
+    }
+
+    /// Try to find an appropriate function for calling a given RPC method on a
+    /// given RPC-visible object.
+    ///
+    /// On success, return a Future.
+    //
+    // TODO RPC: Possibly deprecate this in favor of crate::invoke_rpc_method
+    pub fn invoke(
+        &self,
+        obj: Arc<dyn Object>,
+        method: Box<dyn DynMethod>,
+        ctx: Arc<dyn Context>,
+        sink: BoxedUpdateSink,
+    ) -> Result<RpcResultFuture, InvokeError> {
+        let rpc_invoker = self.rpc_invoker(obj.as_ref(), method.as_ref())?;
+
+        rpc_invoker.invoke(obj, method, ctx, sink)
     }
 
     /// Invoke the given method on `obj` within `ctx`, and return its
@@ -674,19 +714,17 @@ impl DispatchTable {
     ///
     /// Unlike `invoke`, this method does not return a type-erased result,
     /// and does not require that the result can be serialized as an RPC object.
+    //
+    // TODO RPC: Possibly deprecate this in favor of crate::invoke_special_method.
     pub async fn invoke_special<M: crate::Method>(
         &self,
         obj: Arc<dyn Object>,
         method: M,
-        ctx: Box<dyn Context>,
+        ctx: Arc<dyn Context>,
     ) -> Result<Box<Result<M::Output, M::Error>>, InvokeError> {
-        let func_type = FuncType {
-            obj_id: obj.type_id(),
-            method_id: method.type_id(),
-        };
-        let func = self.map.get(&func_type).ok_or(InvokeError::NoImpl)?;
+        let invoker = self.special_invoker::<M>(obj.as_ref())?;
 
-        let fut: SpecialResultFuture = func.invoker.invoke_special(obj, Box::new(method), ctx)?;
+        let fut = invoker.invoke_special(obj, Box::new(method), ctx)?;
         fut.await
             .downcast()
             .map_err(|_| InvokeError::Bug(tor_error::internal!("Downcast to wrong type")))
@@ -694,7 +732,7 @@ impl DispatchTable {
 }
 
 /// An error that occurred while trying to invoke a method on an object.
-#[derive(Debug, thiserror::Error)]
+#[derive(Debug, Clone, thiserror::Error)]
 #[non_exhaustive]
 pub enum InvokeError {
     /// There is no implementation for the given combination of object
@@ -707,6 +745,16 @@ pub enum InvokeError {
     Bug(#[from] tor_error::Bug),
 }
 
+impl tor_error::HasKind for InvokeError {
+    fn kind(&self) -> tor_error::ErrorKind {
+        use tor_error::ErrorKind as EK;
+        match self {
+            InvokeError::NoImpl => EK::RpcMethodNotFound,
+            InvokeError::Bug(e) => e.kind(),
+        }
+    }
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -775,7 +823,7 @@ mod test {
     async fn getname_swan(
         _obj: Arc<Swan>,
         _method: Box<GetName>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "swan".to_string(),
@@ -784,7 +832,7 @@ mod test {
     async fn getname_sheep(
         _obj: Arc<Sheep>,
         _method: Box<GetName>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "sheep".to_string(),
@@ -793,7 +841,7 @@ mod test {
     async fn getname_wombat(
         _obj: Arc<Wombat>,
         _method: Box<GetName>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "wombat".to_string(),
@@ -802,7 +850,7 @@ mod test {
     async fn getname_brick(
         _obj: Arc<Brick>,
         _method: Box<GetName>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "brick".to_string(),
@@ -811,7 +859,7 @@ mod test {
     async fn getkids_swan(
         _obj: Arc<Swan>,
         _method: Box<GetKids>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "cygnets".to_string(),
@@ -820,7 +868,7 @@ mod test {
     async fn getkids_sheep(
         _obj: Arc<Sheep>,
         _method: Box<GetKids>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError> {
         Ok(Outcome {
             v: "lambs".to_string(),
@@ -829,7 +877,7 @@ mod test {
     async fn getkids_wombat(
         _obj: Arc<Wombat>,
         _method: Box<GetKids>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
         mut sink: UpdateSink<String>,
     ) -> Result<Outcome, crate::RpcError> {
         let _ignore = sink.send("brb, burrowing".to_string()).await;
@@ -889,7 +937,7 @@ mod test {
     async fn getname_generic<T, U>(
         obj: Arc<GenericObj<T, U>>,
         _method: Box<GetName>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError>
     where
         T: Send + Sync + 'static + Clone + ToString,
@@ -902,7 +950,7 @@ mod test {
     async fn getkids_generic<T, U>(
         obj: Arc<GenericObj<T, U>>,
         _method: Box<GetKids>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<Outcome, crate::RpcError>
     where
         T: Send + Sync + 'static + Clone + ToString,
@@ -943,7 +991,7 @@ mod test {
         ) -> Result<RpcResultFuture, InvokeError> {
             let animal: Arc<dyn crate::Object> = Arc::new(obj);
             let request: Box<dyn DynMethod> = Box::new(method);
-            let ctx = Box::new(Ctx {});
+            let ctx = Arc::new(Ctx {});
             let discard = Box::pin(futures::sink::drain().sink_err_into());
             table.invoke(animal, request, ctx, discard)
         }
@@ -1037,7 +1085,7 @@ mod test {
     async fn specialonly_swan(
         _obj: Arc<Swan>,
         _method: Box<SpecialOnly>,
-        _ctx: Box<dyn crate::Context>,
+        _ctx: Arc<dyn crate::Context>,
     ) -> Result<MyObject, MyObject> {
         Ok(MyObject {})
     }
@@ -1048,7 +1096,7 @@ mod test {
         let table = crate::DispatchTable::from_inventory();
 
         let res: Outcome = table
-            .invoke_special(Arc::new(Swan), GetKids, Box::new(Ctx {}))
+            .invoke_special(Arc::new(Swan), GetKids, Arc::new(Ctx {}))
             .await
             .unwrap()
             .unwrap();
@@ -1056,7 +1104,7 @@ mod test {
         assert_eq!(res.v, "cygnets");
 
         let _an_obj: MyObject = table
-            .invoke_special(Arc::new(Swan), SpecialOnly {}, Box::new(Ctx {}))
+            .invoke_special(Arc::new(Swan), SpecialOnly {}, Arc::new(Ctx {}))
             .await
             .unwrap()
             .unwrap();
