@@ -16,7 +16,7 @@ define_derive_deftly! {
 ///     and `CoarseTimeProvider`.
 // This could perhaps be further reduced:
 // ambassador might be able to remove most of the body (although does it do async well?)
-    SomeMockRuntime for struct, expect items =
+    SomeMockRuntime for struct, expect items:
 
  $(
   ${when fmeta(mock(task))}
@@ -60,6 +60,9 @@ define_derive_deftly! {
         fn tls_connector(&self) -> Self::Connector {
             self.$fname.tls_connector()
         }
+        fn supports_keying_material_export(&self) -> bool {
+            self.$fname.supports_keying_material_export()
+        }
     }
 
     #[async_trait]
